/**
 * Copyright (c) 2003, Xith3D Project Group
 * All rights reserved.
 *
 * Portions based on the Java3D interface, Copyright by Sun Microsystems.
 * Many thanks to the developers of Java3D and Sun Microsystems for their
 * innovation and design.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the 'Xith3D Project Group' nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) A
 * RISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 */

import java.awt.*;
import java.awt.event.*;

import com.xith3d.scenegraph.*;
import com.xith3d.render.*;
import com.xith3d.render.jogl.*;
import javax.vecmath.*;
import javax.swing.*;

import com.xith3d.image.*;
import com.xith3d.loaders.texture.*;
import com.xith3d.userinterface.UIWindowManager;
import com.xith3d.userinterface.UIWindow;
import com.xith3d.userinterface.UIEventAdapter;
import com.xith3d.test.*;
import com.magicosm.skins.Skin;
import com.magicosm.skins.Theme;
import com.magicosm.skins.SkinLoader;


/**
 * Simple Xith3D Coloring Attributes test
 * 
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: JProof</p>
 * @author YVG
 */
public class Xith3DGuiTest
{

	Transform3D testRotateY = new Transform3D();
	TransformGroup testRotateYGroup = new TransformGroup();
    Material mat;
    UIWindowManager windowMgr;

	protected BranchGroup createSceneGraph() throws Exception
	{
		BranchGroup objRoot = new BranchGroup();

        AmbientLight light = new AmbientLight(true,new Color3f(0,0,1));
        objRoot.addChild(light);

		// Rotate a bit around X axis to see the transfrorm effect
		Transform3D testRotateX = new Transform3D();
		TransformGroup testRotateXGroup = new TransformGroup();
		//		testRotateX.rotX(0.6f);
		testRotateX.rotX(0.0f);
		testRotateXGroup.setTransform(testRotateX);
		objRoot.addChild(testRotateXGroup);

		// Add animation TransformGroup
		testRotateYGroup.setTransform(testRotateY);
		testRotateXGroup.addChild(testRotateYGroup);

		// Make extra transfrom we may want to play with (scale etc.)
		TransformGroup sceneRootTransform = new TransformGroup();
		Transform3D t = new Transform3D();
		t.setIdentity();
		sceneRootTransform.setTransform(t);
		testRotateYGroup.addChild(sceneRootTransform);

		TextureLoader.tf.registerPath(".");
		TextureLoader.tf.registerPath("..");
//		Texture2D texture = (Texture2D) TextureLoader.tf.getMinMapTexture("stone.jpg");

        mat = new Material();
        mat.setAmbientColor(0.75f,0.75f,0.75f);
        mat.setLightingEnable(true);

		Shape3D plane;
		Geometry gPlane;
		Appearance aPlane;
		ColoringAttributes ca;

		plane = new Shape3D();
		gPlane = TestUtils.createNonTexturedPlane(-0.4f, -0.4f, 0f, 0.6f, 0.6f);
		aPlane = new Appearance();
		aPlane.setPolygonAttributes(new PolygonAttributes(PolygonAttributes.POLYGON_FILL, PolygonAttributes.CULL_NONE, 0));
		plane.setAppearance(aPlane);
		plane.setGeometry(gPlane);
		sceneRootTransform.addChild(plane);

		plane = new Shape3D();
		gPlane = TestUtils.createNonTexturedPlane(0.4f, -0.4f, 0f, 0.6f, 0.6f);
		aPlane = new Appearance();
		aPlane.setPolygonAttributes(new PolygonAttributes(PolygonAttributes.POLYGON_FILL, PolygonAttributes.CULL_NONE, 0));
		ca = new ColoringAttributes();
		ca.setColor(new Color3f(1f, 0f, 0f));
		ca.setShadeModel(ColoringAttributes.SHADE_FLAT);
		aPlane.setColoringAttributes(ca);
		plane.setAppearance(aPlane);
		plane.setGeometry(gPlane);
		sceneRootTransform.addChild(plane);

		plane = new Shape3D();
		gPlane = TestUtils.createNonTexturedPlane(-0.4f, 0.4f, 0f, 0.6f, 0.6f);
		aPlane = new Appearance();
		aPlane.setPolygonAttributes(new PolygonAttributes(PolygonAttributes.POLYGON_FILL, PolygonAttributes.CULL_NONE, 0));
		ca = new ColoringAttributes();
		ca.setColor(new Color3f(0f, 1f, 0f));
		ca.setShadeModel(ColoringAttributes.SHADE_FLAT);
		aPlane.setColoringAttributes(ca);
		plane.setAppearance(aPlane);
		plane.setGeometry(gPlane);
		sceneRootTransform.addChild(plane);

		plane = new Shape3D();
		gPlane = TestUtils.createNonTexturedPlane(0.4f, 0.4f, 0f, 0.6f, 0.6f);
		aPlane = new Appearance();
		aPlane.setPolygonAttributes(new PolygonAttributes(PolygonAttributes.POLYGON_FILL, PolygonAttributes.CULL_NONE, 0));
		ca = new ColoringAttributes();
		ca.setColor(new Color3f(0f, 0f, 1f));
		ca.setShadeModel(ColoringAttributes.SHADE_FLAT);
		aPlane.setColoringAttributes(ca);
		plane.setAppearance(aPlane);
		plane.setGeometry(gPlane);
		sceneRootTransform.addChild(plane);

		Shape3D shape = new Shape3D();
		Geometry g = TestUtils.createCubeViaTriangles(0f, 0f, -0.6f, 0.8f, true);
		Appearance a = new Appearance();
		a.setPolygonAttributes(new PolygonAttributes(PolygonAttributes.POLYGON_FILL, PolygonAttributes.CULL_BACK, 0));
//		a.setTexture(texture);
        a.setMaterial(mat);
		shape.setAppearance(a);
		shape.setGeometry(g);
		sceneRootTransform.addChild(shape);

		return objRoot;
	}

	View view;

	public void init() throws Exception
	{
		// Prepare full-screen canvas adapted for current resolution
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		VirtualUniverse universe = new VirtualUniverse();
		view = new View();
		view.setBackClipDistance(100f);
		view.setProjectionPolicy(View.PERSPECTIVE_PROJECTION);
		view.getTransform().lookAt(
			new Vector3f(0, 0, 3),
			new Vector3f(0, 0, 0),
			new Vector3f(0, 1, 3));
		Locale locale = new Locale();
		universe.addLocale(locale);
		universe.addView(view);

		RenderPeer renderPeer = new RenderPeerImpl();
		CanvasPeer canvasPeer =
			renderPeer.makeCanvas(
				null,
				screenSize.width - 200,
				screenSize.height - 200,
				16,
				true);
		canvasPeer.getWindow().setLocation(100, 100);
		Canvas3D canvas = new Canvas3D();
		canvas.set3DPeer(canvasPeer);
		view.addCanvas3D(canvas);

        // construct a window manager for this canvas

        windowMgr = new UIWindowManager(canvas);

        // load the skin

        Skin skin = SkinLoader.loadFromXML("default");
        TestWindow window = new TestWindow(skin,250,400,true,true);
        windowMgr.addOverlay(window);
        window.setPosition(10,10);
        window.setVisible(true);

        UIEventAdapter eventAdapter = new UIEventAdapter(windowMgr);
        canvas.get3DPeer().getComponent().addKeyListener(eventAdapter);
        canvas.get3DPeer().getComponent().addMouseListener(eventAdapter);
        canvas.get3DPeer().getComponent().addMouseMotionListener(eventAdapter);
        canvas.get3DPeer().getComponent().setFocusable(true);

		BranchGroup scene = createSceneGraph();
		locale.addBranchGraph(scene);

		runTest();
	}


	public void runTest()
	{
		float angle = 0f;

		while (true)
		{
			view.renderOnce();
			try
			{
				Thread.sleep(10L);
			}
			catch (Exception e)
			{
			}
			angle += 0.005f;
			testRotateY.rotY(angle);
			testRotateYGroup.setTransform(testRotateY);
		}
	}


	public static void main(String[] args)
	{
		System.out.println("Hit SPACE to toggle projection policy, or ESC to exit");
		try
		{
			Xith3DGuiTest test = new Xith3DGuiTest();
			test.init();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

}
