/**
 * Copyright (c) 2003, David J. Yazel
 *
 * The Magicosm Development Group is granted the rights to
 * use this code in the development of the Magicosm game and all
 * subsequent derrivations in perpetuity.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) A
 * RISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 */
package com.magicosm.skins;

import com.magicosm.skins.renderers.CappedBoxDef;

import java.awt.*;
import java.awt.image.BufferedImage;


/**
 * Insert package comments here
 * <p>
 * Originally Coded by David Yazel on Oct 12, 2003 at 1:44:05 PM.
 */
public class CappedBoxHelper implements SkinPainter {
    BufferedImage image;
    CappedBoxDef def;

    public CappedBoxHelper(Skin skin, CappedBoxDef def) {
        this.def = def;
        image = skin.getImage(def.getImageName());
    }

    public CappedBoxDef getDef() {
        return def;
    }

    public void paint(Graphics g, int x, int y, int width, int height) {
        if (def.isVertical()) {
            if (height < ((def.getOffset() * 2) + 2)) {
                g.drawImage(image, x, y, width, height, null);
            } else {
                int offset = (int) (((float) def.getOffset() * (float) height) / (float) image.getHeight());
                int h1 = offset;
                int h2 = height - (2 * offset);
                int h3 = offset;

                g.drawImage(image, x, y, x + width, y + h1, 0, 0,
                    image.getWidth(), def.getOffset(), null);
                g.drawImage(image, x, y + h1, x + width, y + h1 + h2, 0,
                    def.getOffset(), image.getWidth(), def.getOffset() + 20,
                    null);
                g.drawImage(image, x, y + h1 + h2, x + width, y + h1 + h2 + h3,
                    0, image.getHeight() - def.getOffset(), image.getWidth(),
                    image.getHeight(), null);
            }
        } else {
            if (width < ((def.getOffset() * 2) + 2)) {
                g.drawImage(image, x, y, width, height, null);
            } else {
                int offset = (int) (((float) def.getOffset() * (float) height) / (float) image.getHeight());
                int h1 = offset;
                int h2 = width - (2 * offset);
                int h3 = offset;

                ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
                ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_INTERPOLATION,
                    RenderingHints.VALUE_INTERPOLATION_BICUBIC);

                g.drawImage(image, x, y, x + h1, y + height, 0, 0,
                    def.getOffset(), image.getHeight(), null);
                g.drawImage(image, x + h1, y, x + h1 + h2, y + height,
                    def.getOffset(), 0, def.getOffset() + 20,
                    image.getHeight(), null);
                g.drawImage(image, x + h1 + h2, y, x + h1 + h2 + h3,
                    y + height, image.getWidth() - def.getOffset(), 0,
                    image.getWidth(), image.getHeight(), null);
            }
        }
    }
}
