package com.magicosm.skins;

import com.xith3d.utility.image.DrawingTools;
import com.xith3d.utility.image.ImageUtility;
import com.xith3d.utility.image.MaskedBackgroundBuilder;

import java.awt.*;
import java.awt.image.*;

import javax.swing.*;
import javax.swing.border.*;


/**
 * <p>The component helper is used to draw complex backgrounds and graphics
 * in the special cosm swing system</p>
 * <p> </p>
 * <p> Copyright (c) 2000-2003, David J. Yazel</p>
 * <p> Teseract Software, LLP</p>
 * @author David Yazel
 *
 */
public class ComponentHelper implements Icon, SkinPainter {
    boolean dirty = false;
    MaskedBackgroundBuilder builder = null;
    int cornerSize = 20;

    // cached images
    BufferedImage backImage = null;
    int w;
    int h;

    public ComponentHelper(BufferedImage borderImg,
        BufferedImage borderMaskImg, BufferedImage centerImg, int cornerSize) {
        this.cornerSize = cornerSize;

        builder = new MaskedBackgroundBuilder(borderImg, borderMaskImg,
                centerImg);
        builder.setCornerSize(cornerSize);
        backImage = null;
    }

    public ComponentHelper() {
    }

    public int getIconHeight() {
        return h;
    }

    public int getIconWidth() {
        return w;
    }

    public void paintIcon(Component c, Graphics g, int x, int y) {
        paint(g, 0, 0, c.getWidth(), c.getHeight());
    }

    public void paint(Graphics g, int x, int y, int width, int height) {
        if (builder == null) {
            g.setColor(Color.red);
            g.fillRect(x, y, width, height);
        } else {
            if ((backImage == null) ||
                    ((width != backImage.getWidth()) ||
                    (height != backImage.getHeight()))) {
                backImage = builder.getImage(width, height, true);
            }

            g.drawImage(backImage, x, y, null);
        }
    }

    public void setBuilder(MaskedBackgroundBuilder builder) {
        this.builder = builder;
        dirty = true;
    }

    public int getCornerSize() {
        return cornerSize;
    }

    public void setCornerSize(int cornerSize) {
        this.cornerSize = cornerSize;
        dirty = true;
    }
}
