/**
 * Copyright (c) 2003, Xith3D Project Group
 * All rights reserved.
 *
 * Portions based on the Java3D interface, Copyright by Sun Microsystems.
 * Many thanks to the developers of Java3D and Sun Microsystems for their
 * innovation and design.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the 'Xith3D Project Group' nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) A
 * RISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 */
package com.magicosm.skins;

import com.magicosm.skins.renderers.BoxDef;
import com.magicosm.skins.renderers.CappedBoxDef;
import com.magicosm.skins.renderers.FontDef;
import com.magicosm.skins.renderers.MaskedBoxDef;
import com.magicosm.skins.styles.*;

import com.xith3d.utility.logs.Log;

import org.xml.sax.*;

import java.awt.*;

import java.io.*;

import java.net.URI;
import java.net.URL;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import javax.xml.parsers.SAXParserFactory;


/**
 * Loads the skin information from XML and assembles the nodes which
 * control rendering
 * <p>
 * Originally Coded by David Yazel on Oct 6, 2003 at 6:37:46 PM.
 */
public class SkinLoader {
    public static Skin loadFromXML(String name) {
        final Skin skin = new Skin("com/magicosm/skins/" + name + "/");

        ClassLoader classloader = ClassLoader.getSystemClassLoader();
        URL url = classloader.getResource("com/magicosm/skins/" + name +
                "/skin.xml");

        if (url == null) {
            throw new Error("Cannot find skin definition : " +
                "com/magicosm/skins/" + name + "/skin.xml");
        }

        try {
            SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
            saxParserFactory.setNamespaceAware(false);
            saxParserFactory.setValidating(false);

            javax.xml.parsers.SAXParser saxParser = saxParserFactory.newSAXParser();

            XMLReader reader = saxParser.getXMLReader();
            reader.setContentHandler(new org.xml.sax.helpers.DefaultHandler() {
                    Locator locator;
                    Theme curTheme;

                    public void startElement(String namespaceURI,
                        String localName, String qName, Attributes attribs)
                        throws SAXException {
                        if (qName.equals("skin")) {
                            skin.setName(attribs.getValue("name"));
                            skin.setAuthor(attribs.getValue("author"));
                            skin.setVersion(attribs.getValue("version"));
                        } else if (qName.equals("theme")) {
                            curTheme = new Theme(attribs.getValue("name"),
                                    attribs.getValue("extends"));

                            try {
                                skin.addTheme(curTheme);
                            } catch (IOException e) {
                                System.err.println(
                                    "Invalid Theme Definition (line " +
                                    locator.getLineNumber() + ")" +
                                    e.getMessage());
                            }
                        } else if (qName.equals("component")) {
                            String className = attribs.getValue("class");
                            String styleName = attribs.getValue("style");

                            try {
                                curTheme.add(skin, className, styleName);
                            } catch (Exception e) {
                                System.err.println(
                                    "Invalid Theme Definition (line " +
                                    locator.getLineNumber() + ")" +
                                    e.getMessage());
                            }
                        } else if (qName.equals("masked-box")) {
                            MaskedBoxDef box = new MaskedBoxDef();
                            box.setName(attribs.getValue("name"));
                            box.setBorderMaskFile(attribs.getValue("mask"));
                            box.setBorderDisableFile(attribs.getValue(
                                    "borderDisable"));
                            box.setBorderFile(attribs.getValue("border"));
                            box.setCenterDisableFile(attribs.getValue(
                                    "centerDisable"));
                            box.setCenterFile(attribs.getValue("centerNormal"));
                            box.setCenterHighlightFile(attribs.getValue(
                                    "centerHighlight"));
                            box.setCornerSize(Integer.parseInt(attribs.getValue(
                                        "maskCorner")));

                            try {
                                skin.addRenderer(box);
                                Log.print("loaded masked box def " +
                                    box.getName());
                            } catch (IOException e) {
                                System.err.println(
                                    "Invalid Box Definition (line " +
                                    locator.getLineNumber() + ")" +
                                    e.getMessage());
                            }
                        } else if (qName.equals("box")) {
                            BoxDef box = new BoxDef();
                            box.setName(attribs.getValue("name"));
                            box.setImageDisableFile(attribs.getValue(
                                    "imageDisable"));
                            box.setImageFile(attribs.getValue("image"));
                            box.setImageHighlightFile(attribs.getValue(
                                    "imageHighlight"));

                            String stretch = attribs.getValue("stretch");

                            if (stretch.equalsIgnoreCase("true")) {
                                box.setStretch(true);
                            } else {
                                box.setStretch(false);
                            }

                            try {
                                skin.addRenderer(box);
                                Log.print("loaded box def " + box.getName());
                            } catch (IOException e) {
                                System.err.println(
                                    "Invalid Box Definition (line " +
                                    locator.getLineNumber() + ")" +
                                    e.getMessage());
                            }
                        } else if (qName.equals("capped-box")) {
                            CappedBoxDef box = new CappedBoxDef();
                            box.setName(attribs.getValue("name"));
                            box.setImageName(attribs.getValue("image"));
                            box.setOffset(Integer.parseInt(attribs.getValue(
                                        "offset")));

                            String orientation = attribs.getValue("orientation");

                            if (orientation.equalsIgnoreCase("HORIZONTAL")) {
                                box.setVertical(false);
                            } else {
                                box.setVertical(true);
                            }

                            try {
                                skin.addRenderer(box);
                                Log.print("loaded capped box def " +
                                    box.getName());
                            } catch (IOException e) {
                                System.err.println(
                                    "Invalid Box Definition (line " +
                                    locator.getLineNumber() + ")" +
                                    e.getMessage());
                            }
                        } else if (qName.equals("checkbox") ||
                                qName.equals("panel") ||
                                qName.equals("togglebutton") ||
                                qName.equals("radiobutton") ||
                                qName.equals("button") ||
                                qName.equals("textfield") ||
                                qName.equals("label") ||
                                qName.equals("tabbedpane") ||
                                qName.equals("scrollpane")) {
                            ControlStyle button = new ControlStyle();
                            button.setName(attribs.getValue("name"));
                            button.setFontRenderer(attribs.getValue("font"));
                            button.setBackgroundRenderer(attribs.getValue(
                                    "background"));

                            try {
                                skin.addStyle(button);
                                Log.print("loaded style " + button.getName());
                            } catch (IOException e) {
                                System.err.println("Invalid Definition (line " +
                                    locator.getLineNumber() + ")" +
                                    e.getMessage());
                            }
                        } else if (qName.equals("frame")) {
                            FrameStyle button = new FrameStyle();
                            button.setName(attribs.getValue("name"));
                            button.setFontRenderer(attribs.getValue("font"));
                            button.setBackgroundRenderer(attribs.getValue(
                                    "background"));

                            try {
                                button.setTitle(attribs.getValue("title"));
                            } catch (Throwable e) {
                            }

                            try {
                                skin.addStyle(button);
                                Log.print("loaded style " + button.getName());
                            } catch (IOException e) {
                                System.err.println("Invalid Definition (line " +
                                    locator.getLineNumber() + ")" +
                                    e.getMessage());
                            }
                        } else if (qName.equals("scrollbar")) {
                            ScrollbarStyle control = new ScrollbarStyle();
                            control.setName(attribs.getValue("name"));
                            control.setVthumb(attribs.getValue("vthumb"));
                            control.setHthumb(attribs.getValue("hthumb"));
                            control.setVtrough(attribs.getValue("vtrough"));
                            control.setHtrough(attribs.getValue("htrough"));
                            control.setLeft(attribs.getValue("left"));
                            control.setRight(attribs.getValue("right"));
                            control.setUp(attribs.getValue("up"));
                            control.setDown(attribs.getValue("down"));

                            try {
                                skin.addStyle(control);
                                Log.print("loaded style " + control.getName());
                            } catch (IOException e) {
                                System.err.println("Invalid Definition (line " +
                                    locator.getLineNumber() + ")" +
                                    e.getMessage());
                            }
                        } else if (qName.equals("font")) {
                            FontDef fontDef = new FontDef();
                            fontDef.setName(attribs.getValue("name"));
                            fontDef.setFont(attribs.getValue("font"));
                            fontDef.setSize(Integer.parseInt(attribs.getValue(
                                        "size")));

                            String colorStr = attribs.getValue("color");
                            String[] rgb = colorStr.split(",");

                            if (rgb.length != 3) {
                                System.err.println(
                                    "Invalid color Definition (line " +
                                    locator.getLineNumber() + ")");
                            }

                            try {
                                Color c = null;

                                if (rgb[0].indexOf(".") >= 0) {
                                    c = new Color(Float.parseFloat(rgb[0]),
                                            Float.parseFloat(rgb[1]),
                                            Float.parseFloat(rgb[2]));
                                    fontDef.setColor(c);
                                } else {
                                    c = new Color(Integer.parseInt(rgb[0]),
                                            Integer.parseInt(rgb[1]),
                                            Integer.parseInt(rgb[2]));
                                    fontDef.setColor(c);
                                }
                            } catch (NumberFormatException e) {
                                System.err.println("Invalid color Definition " +
                                    rgb[0] + "," + rgb[1] + "," + rgb[2] +
                                    "  (line " + locator.getLineNumber() +
                                    ") " + e.getMessage());
                            }

                            String style = attribs.getValue("style");

                            if (style == null) {
                                fontDef.setStyle(Font.PLAIN);
                            } else if (style.equalsIgnoreCase("PLAIN")) {
                                fontDef.setStyle(Font.PLAIN);
                            } else if (style.equalsIgnoreCase("ITALIC")) {
                                fontDef.setStyle(Font.ITALIC);
                            } else if (style.equalsIgnoreCase("BOLD")) {
                                fontDef.setStyle(Font.BOLD);
                            }

                            try {
                                skin.addRenderer(fontDef);
                                Log.print("loaded font def " +
                                    fontDef.getName());
                            } catch (IOException e) {
                                System.err.println(
                                    "Invalid Box Definition (line " +
                                    locator.getLineNumber() + ")" +
                                    e.getMessage());
                            }
                        }
                    }

                    public void characters(char[] ch, int start, int length) {
                        /* not needed yet */
                    }

                    public void setDocumentLocator(Locator locator) {
                        super.setDocumentLocator(locator);
                        this.locator = locator;
                    }
                });
            reader.parse(new org.xml.sax.InputSource(url.openStream()));
        } catch (Exception e) {
            Log.print(e);
        }

        return skin;
    }
}
