/**
 * Copyright (c) 2003, David J. Yazel
 *
 * The Magicosm Development Group is granted the rights to
 * use this code in the development of the Magicosm game and all
 * subsequent derrivations in perpetuity.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) A
 * RISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 */
package com.magicosm.skins;

import com.magicosm.skins.renderers.BaseDef;
import com.magicosm.skins.renderers.BoxDef;
import com.magicosm.skins.renderers.FontDef;
import com.magicosm.skins.renderers.MaskedBoxDef;
import com.magicosm.skins.styles.BaseStyle;
import com.magicosm.skins.styles.ControlStyle;

import com.xith3d.utility.image.DrawingTools;

import java.awt.*;

import javax.swing.*;


/**
 * Insert package comments here
 * <p>
 * Originally Coded by David Yazel on Oct 9, 2003 at 8:43:18 PM.
 */
public class SkinRadioButton extends JRadioButton
    implements SkinComponentInterface {
    SkinPainter helper;
    SkinPainter highlightHelper;

    public void setStyle(Skin skin, BaseStyle style) {
        if (style instanceof ControlStyle) {
            ControlStyle cs = (ControlStyle) style;
            BaseDef background = cs.getBackgroundDef();
            Font font = cs.getFont();

            if (background instanceof MaskedBoxDef) {
                MaskedBoxDef def = (MaskedBoxDef) background;
                helper = style.getPainter(BaseStyle.STYLE_NORMAL, skin, def);
                highlightHelper = style.getPainter(BaseStyle.STYLE_HIGHLIGHT,
                        skin, def);
            } else if (background instanceof BoxDef) {
                BoxDef def = (BoxDef) background;
                helper = new BoxHelper(skin.getImage(def.getImageFile()), false);
                highlightHelper = new BoxHelper(skin.getImage(
                            def.getImageHighlightFile()), false);
            }

            setMargin(new Insets(0, 0, 0, 0));
            setBorder(null);

            setOpaque(false);
            setContentAreaFilled(false);
            setIconTextGap(0);
            setFont(font);
            setForeground(new Color(237, 198, 23));
        }
    }

    public void paint(Graphics g) {
        SkinPainter h = null;

        if (isSelected()) {
            h = highlightHelper;
        } else {
            h = helper;
        }

        if (h != null) {
            h.paint(g, 0, 0, getWidth(), getHeight());

            ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
            ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

            DrawingTools.drawText(new Rectangle(14, 0, getWidth(), getHeight()),
                g, getText());
        } else {
            super.paint(g);
        }
    }
}
