/**
 * Copyright (c) 2003, David J. Yazel
 *
 * The Magicosm Development Group is granted the rights to
 * use this code in the development of the Magicosm game and all
 * subsequent derrivations in perpetuity.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) A
 * RISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 */
package com.magicosm.skins;

import com.magicosm.skins.renderers.BaseDef;
import com.magicosm.skins.renderers.CappedBoxDef;
import com.magicosm.skins.renderers.FontDef;
import com.magicosm.skins.renderers.MaskedBoxDef;
import com.magicosm.skins.styles.BaseStyle;
import com.magicosm.skins.styles.ControlStyle;
import com.magicosm.skins.styles.ScrollbarStyle;

import com.xith3d.utility.image.DrawingTools;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.EmptyBorder;


/**
 * Insert package comments here
 * <p>
 * Originally Coded by David Yazel on Oct 9, 2003 at 8:43:18 PM.
 */
public class SkinScrollbar extends JScrollBar implements SkinComponentInterface {
    SkinPainter troughHelper;
    SkinPainter thumbHelper;
    SkinPainter backHelper;
    SkinPainter forwardHelper;
    int buttonSize;

    public void setStyle(Skin skin, BaseStyle style) {
        if (style instanceof ScrollbarStyle) {
            ScrollbarStyle cs = (ScrollbarStyle) style;

            if (this.getOrientation() == JScrollBar.VERTICAL) {
                troughHelper = cs.getPainter(BaseStyle.STYLE_NORMAL, skin,
                        cs.getVtroughDef());
                thumbHelper = cs.getPainter(BaseStyle.STYLE_NORMAL, skin,
                        cs.getVthumbDef());
                backHelper = cs.getPainter(BaseStyle.STYLE_NORMAL, skin,
                        cs.getUpDef());
                forwardHelper = cs.getPainter(BaseStyle.STYLE_NORMAL, skin,
                        cs.getDownDef());
            } else {
                troughHelper = cs.getPainter(BaseStyle.STYLE_NORMAL, skin,
                        cs.getHtroughDef());
                thumbHelper = cs.getPainter(BaseStyle.STYLE_NORMAL, skin,
                        cs.getHthumbDef());
                backHelper = cs.getPainter(BaseStyle.STYLE_NORMAL, skin,
                        cs.getLeftDef());
                forwardHelper = cs.getPainter(BaseStyle.STYLE_NORMAL, skin,
                        cs.getRightDef());
            }

            setOpaque(false);
            setForeground(new Color(237, 198, 23));
        }

        Component[] cs = this.getComponents();

        for (int i = 0; i < cs.length; i++) {
            cs[i].setVisible(false);

            if (getOrientation() == JScrollBar.VERTICAL) {
                buttonSize = (int) cs[i].getPreferredSize().getHeight();
            } else {
                buttonSize = (int) cs[i].getPreferredSize().getWidth();
            }
        }

        System.out.println("size is " + buttonSize);
    }

    public void paint(Graphics g) {
        if (troughHelper != null) {
            ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
            ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

            if (getOrientation() == JScrollBar.VERTICAL) {
                troughHelper.paint(g, 0, 0, getWidth(), getHeight());
                backHelper.paint(g, 0, 0, getWidth(), buttonSize);
                forwardHelper.paint(g, 0, getHeight() - buttonSize, getWidth(),
                    buttonSize);

                float percent = (float) getVisibleAmount() / ((float) this.getMaximum() -
                    (float) getMinimum());
                int size = getHeight() - (2 * buttonSize);
                size *= percent;

                float ppercent = (float) getValue() / ((float) this.getMaximum() -
                    (float) getMinimum());
                int psize = getHeight() - (2 * buttonSize);
                psize *= ppercent;

                thumbHelper.paint(g, 2, buttonSize + psize, getWidth() - 4, size);
            } else {
                troughHelper.paint(g, 0, 0, getWidth(), getHeight());
                backHelper.paint(g, 0, 0, buttonSize, getHeight());
                forwardHelper.paint(g, getWidth() - buttonSize, 0, buttonSize,
                    getHeight());

                float percent = (float) getVisibleAmount() / ((float) this.getMaximum() -
                    (float) getMinimum());
                int size = getWidth() - (2 * buttonSize);
                size *= percent;

                float ppercent = (float) getValue() / ((float) this.getMaximum() -
                    (float) getMinimum());
                int psize = getWidth() - (2 * buttonSize);
                psize *= ppercent;

                thumbHelper.paint(g, buttonSize + psize, 2, size,
                    getHeight() - 4);
            }
        } else {
            super.paint(g);
        }
    }
}
