/**
 * Copyright (c) 2003, David J. Yazel
 *
 * The Magicosm Development Group is granted the rights to
 * use this code in the development of the Magicosm game and all
 * subsequent derrivations in perpetuity.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) A
 * RISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 */
package com.magicosm.skins;

import com.magicosm.skins.renderers.BaseDef;
import com.magicosm.skins.renderers.MaskedBoxDef;
import com.magicosm.skins.styles.BaseStyle;
import com.magicosm.skins.styles.ControlStyle;

import com.xith3d.utility.image.DrawingTools;

import java.awt.*;

import javax.swing.*;


/**
 * Insert package comments here
 * <p>
 * Originally Coded by David Yazel on Oct 9, 2003 at 11:34:55 PM.
 */
public class SkinTabbedPane extends JTabbedPane
    implements SkinComponentInterface {
    SkinPainter[] helper;
    SkinPainter[] helperHighlight;

    public void setStyle(Skin skin, BaseStyle style) {
        if (style instanceof ControlStyle) {
            ControlStyle cs = (ControlStyle) style;
            BaseDef background = cs.getBackgroundDef();
            Font font = cs.getFont();

            if (background instanceof MaskedBoxDef) {
                MaskedBoxDef def = (MaskedBoxDef) background;
                helper = new SkinPainter[this.getTabCount()];
                helperHighlight = new SkinPainter[this.getTabCount()];

                for (int i = 0; i < helper.length; i++) {
                    helper[i] = style.getPainter(BaseStyle.STYLE_NORMAL, skin,
                            def);
                    helperHighlight[i] = style.getPainter(BaseStyle.STYLE_DISABLE,
                            skin, def);
                }

                setBorder(null);

                setOpaque(false);
                setFont(font);
                setForeground(cs.getForeground());
            }
        }
    }

    public void paintComponent(Graphics g) {
        g.setColor(Color.red);

        ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
        ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
            RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

        Component[] cs = getComponents();

        if (helper != null) {
            for (int i = 0; i < helper.length; i++) {
                Rectangle bounds = this.getBoundsAt(i);

                if (this.getSelectedIndex() == i) {
                    helperHighlight[i].paint(g, bounds.x, bounds.y,
                        bounds.width, bounds.height + 2);

                    if (!cs[i].isVisible()) {
                        cs[i].setVisible(true);
                    }
                } else {
                    helper[i].paint(g, bounds.x, bounds.y, bounds.width,
                        bounds.height + 2);

                    if (cs[i].isVisible()) {
                        cs[i].setVisible(false);
                    }
                }

                g.setFont(getFont());
                g.setColor(getForeground());
                DrawingTools.centerText(bounds, g, this.getTitleAt(i));
            }
        } else {
            super.paintComponent(g);
        }
    }
}
