/**
 * Copyright (c) 2003, David J. Yazel
 *
 * The Magicosm Development Group is granted the rights to
 * use this code in the development of the Magicosm game and all
 * subsequent derrivations in perpetuity.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) A
 * RISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 */
package com.magicosm.skins;

import com.magicosm.skins.styles.BaseStyle;
import com.magicosm.skins.styles.ControlStyle;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;


/**
 * Insert package comments here
 * <p>
 * Originally Coded by David Yazel on Oct 9, 2003 at 8:43:18 PM.
 */
public class SkinTextField extends JTextField implements SkinComponentInterface {
    SkinPainter helper;
    SkinPainter disableHelper;

    public void setStyle(Skin skin, BaseStyle style) {
        if (style instanceof ControlStyle) {
            ControlStyle cs = (ControlStyle) style;
            helper = cs.getPainter(BaseStyle.STYLE_NORMAL, skin,
                    cs.getBackgroundDef());
            disableHelper = cs.getPainter(BaseStyle.STYLE_DISABLE, skin,
                    cs.getBackgroundDef());
            setOpaque(false);

            Font font = cs.getFont();
            setFont(font);
            setForeground(cs.getForeground());
            setBorder(null);
        }
    }

    public void paint(Graphics g) {
        SkinPainter h = null;

        if (isEnabled()) {
            h = helper;
        } else {
            h = disableHelper;
        }

        if (h != null) {
            Border b = getBorder();

            if (b instanceof EmptyBorder) {
                EmptyBorder eb = (EmptyBorder) b;
                Insets i = eb.getBorderInsets();
                h.paint(g, i.left, i.top, getWidth() - i.left - i.right,
                    getHeight() - i.top - i.bottom);
            } else {
                h.paint(g, 0, 0, getWidth(), getHeight());
            }
        }

        ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
        ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
            RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);

        super.paint(g);
    }
}
