/**
 * Copyright (c) 2003, David J. Yazel
 *
 * The Magicosm Development Group is granted the rights to
 * use this code in the development of the Magicosm game and all
 * subsequent derrivations in perpetuity.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) A
 * RISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 */
package com.magicosm.skins.styles;

import com.magicosm.skins.*;
import com.magicosm.skins.renderers.BaseDef;
import com.magicosm.skins.renderers.BoxDef;
import com.magicosm.skins.renderers.CappedBoxDef;
import com.magicosm.skins.renderers.MaskedBoxDef;

import java.io.IOException;


/**
 * Insert package comments here
 * <p>
 * Originally Coded by David Yazel on Oct 8, 2003 at 11:33:20 PM.
 */
public class BaseStyle {
    public static final int STYLE_NORMAL = 1;
    public static final int STYLE_DISABLE = 2;
    public static final int STYLE_HIGHLIGHT = 3;

    BaseDef getRenderer(Skin skin, String name, String value)
        throws IOException {
        if (value == null) {
            throw new IOException("attribute " + name + " not defined ");
        }

        BaseDef def = skin.getRenderer(value);

        if (def == null) {
            throw new IOException("Unknown renderer " + value);
        }

        return def;
    }

    BaseDef getRendererOrNull(Skin skin, String name, String value)
        throws IOException {
        if (value == null) {
            throw new IOException("attribute " + name + " not defined ");
        }

        BaseDef def = skin.getRenderer(value);

        if (def == null) {
            return null;
        }

        return def;
    }

    public SkinPainter getPainter(int style, Skin skin, BaseDef def) {
        if (def instanceof BoxDef) {
            BoxDef d = (BoxDef) def;

            switch (style) {
            case STYLE_NORMAL:
                return new BoxHelper(skin.getImage(d.getImageFile()),
                    d.isStretch());

            case STYLE_DISABLE:
                return new BoxHelper(skin.getImage(d.getImageDisableFile()),
                    d.isStretch());

            case STYLE_HIGHLIGHT:
                return new BoxHelper(skin.getImage(d.getImageHighlightFile()),
                    d.isStretch());
            }
        } else if (def instanceof CappedBoxDef) {
            CappedBoxDef d = (CappedBoxDef) def;

            return new CappedBoxHelper(skin, d);
        } else if (def instanceof MaskedBoxDef) {
            MaskedBoxDef d = (MaskedBoxDef) def;

            switch (style) {
            case STYLE_NORMAL:
                return new ComponentHelper(skin.getImage(d.getBorderFile()),
                    skin.getImage(d.getBorderMaskFile()),
                    skin.getImage(d.getCenterFile()), d.getCornerSize());

            case STYLE_DISABLE:
                return new ComponentHelper(skin.getImage(
                        d.getBorderDisableFile()),
                    skin.getImage(d.getBorderMaskFile()),
                    skin.getImage(d.getCenterDisableFile()), d.getCornerSize());

            case STYLE_HIGHLIGHT:
                return new ComponentHelper(skin.getImage(d.getBorderFile()),
                    skin.getImage(d.getBorderMaskFile()),
                    skin.getImage(d.getCenterHighlightFile()), d.getCornerSize());
            }
        }

        return null;
    }
}
