/**
 * Copyright (c) 2003, David J. Yazel
 *
 * The Magicosm Development Group is granted the rights to
 * use this code in the development of the Magicosm game and all
 * subsequent derrivations in perpetuity.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) A
 * RISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 */
package com.magicosm.skins.styles;

import com.magicosm.skins.Skin;
import com.magicosm.skins.SkinNodeInterface;
import com.magicosm.skins.renderers.BaseDef;
import com.magicosm.skins.renderers.FontDef;

import java.awt.*;

import java.io.IOException;


/**
 * Base def for most control styles
 * <p>
 * Originally Coded by David Yazel on Oct 8, 2003 at 11:25:59 PM.
 */
public class ControlStyle extends BaseStyle implements SkinNodeInterface {
    String name;
    String backgroundRenderer;
    String fontRenderer;
    BaseDef backgroundDef;
    Font font;
    Color foreground;

    public Color getForeground() {
        return foreground;
    }

    public void setForeground(Color foreground) {
        this.foreground = foreground;
    }

    public String getBackgroundRenderer() {
        return backgroundRenderer;
    }

    public void setBackgroundRenderer(String backgroundRenderer) {
        this.backgroundRenderer = backgroundRenderer;
    }

    public String getFontRenderer() {
        return fontRenderer;
    }

    public void setFontRenderer(String fontRenderer) {
        this.fontRenderer = fontRenderer;
    }

    public String getName() {
        return name;
    }

    public BaseDef getBackgroundDef() {
        return backgroundDef;
    }

    public void setBackgroundDef(BaseDef backgroundDef) {
        this.backgroundDef = backgroundDef;
    }

    public Font getFont() {
        return font;
    }

    public void setFont(Font font) {
        this.font = font;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void validateAndLoad(Skin skin) throws IOException {
        if (name == null) {
            throw new IOException("element 'name' missing");
        }

        if (backgroundRenderer != null) {
            backgroundDef = skin.getRenderer(backgroundRenderer);

            if (backgroundDef == null) {
                throw new IOException("No such renderer : " +
                    backgroundRenderer);
            }
        }

        if (fontRenderer != null) {
            FontDef f = (FontDef) skin.getRenderer(fontRenderer);
            foreground = f.getColor();
            font = skin.getFont(fontRenderer);

            if (font == null) {
                throw new IOException("No such font defined : " + fontRenderer);
            }
        }
    }
}
